@@ -57,7 +57,7 @@ module IcdGenerator
                          "clGetGLContextInfoKHR", "clUnloadCompiler",
     "clCreateContext", "clCreateContextFromType", "clWaitForEvents"]
   $header_files = ["/usr/include/CL/cl.h", "/usr/include/CL/cl_gl.h", "/usr/include/CL/cl_egl.h",
-    "/usr/include/CL/cl_ext.h", "/usr/include/CL/cl_gl_ext.h"]
+    "/usr/include/CL/cl_ext.h"]
   $windows_header_files = ["/usr/include/CL/cl_dx9_media_sharing.h", "/usr/include/CL/cl_d3d11.h", "/usr/include/CL/cl_d3d10.h"]
   $cl_data_type_error = { "cl_platform_id"   => "CL_INVALID_PLATFORM",
                           "cl_device_id"     => "CL_INVALID_DEVICE",
@@ -209,9 +209,15 @@ EOF
     libdummy_icd_structures += "};\n\n"
     libdummy_icd_structures += "#pragma GCC visibility push(hidden)\n\n"
     libdummy_icd_structures += "extern struct _cl_icd_dispatch master_dispatch; \n\n"
+    libdummy_icd_structures += "#if defined(__APPLE__) || defined(__MACOSX)\n"
+    $use_name_in_test.each { |k, f|
+      libdummy_icd_structures += "#define INT#{f} #{f}\n"
+    }
+    libdummy_icd_structures += "#else\n"
     $use_name_in_test.each { |k, f|
       libdummy_icd_structures += "typeof(#{f}) INT#{f};\n"
     }
+    libdummy_icd_structures += "#endif\n"
     libdummy_icd_structures += "#pragma GCC visibility pop\n\n"
     return libdummy_icd_structures
   end
@@ -246,7 +252,7 @@ EOF
     run_dummy_icd += "\n\n"
     $api_entries.each_key { |func_name|
        next if $forbidden_funcs.include?(func_name)
-       run_dummy_icd += $api_entries[func_name]+";\n"
+       run_dummy_icd += $api_entries[func_name]+"\n"
     }
     run_dummy_icd += "\n\n"
     run_dummy_icd += "void call_all_OpenCL_functions(cl_platform_id chosen_platform) {\n"
@@ -344,7 +350,20 @@ EOF
     icd_layer_source = "/**\n#{$license}\n*/\n"
     icd_layer_source += <<EOF
 #include <stdio.h>
-#include "ocl_icd_layer.h"
+#define CL_USE_DEPRECATED_OPENCL_1_0_APIS
+#define CL_USE_DEPRECATED_OPENCL_1_1_APIS
+#define CL_USE_DEPRECATED_OPENCL_1_2_APIS
+#define CL_USE_DEPRECATED_OPENCL_2_0_APIS
+#define CL_USE_DEPRECATED_OPENCL_2_1_APIS
+#define CL_USE_DEPRECATED_OPENCL_2_2_APIS
+#define CL_TARGET_OPENCL_VERSION 300
+#ifdef HAVE_OPENCL_CL_LAYER_H
+#include <OpenCL/cl_layer.h>
+#elif defined HAVE_CL_CL_LAYER_H
+#include <CL/cl_layer.h>
+#else
+#include "khronos-headers/CL/cl_layer.h"
+#endif
 
 static struct _cl_icd_dispatch dispatch = {NULL};
 static const struct _cl_icd_dispatch *tdispatch;
@@ -382,14 +401,14 @@ clInitLayer(
     const struct _cl_icd_dispatch  *target_dispatch,
     cl_uint                        *num_entries_out,
     const struct _cl_icd_dispatch **layer_dispatch) {
-  if (!target_dispatch || !layer_dispatch ||!num_entries_out || num_entries < OCL_ICD_LAST_FUNCTION+1)
+  if (!target_dispatch || !layer_dispatch ||!num_entries_out || num_entries < sizeof(dispatch)/sizeof(dispatch.clGetPlatformIDs))
     return -1;
 
   _init_dispatch();
 
   tdispatch = target_dispatch;
   *layer_dispatch = &dispatch;
-  *num_entries_out = OCL_ICD_LAST_FUNCTION+1;
+  *num_entries_out = sizeof(dispatch)/sizeof(dispatch.clGetPlatformIDs);
   return CL_SUCCESS;
 }
 
@@ -683,11 +702,15 @@ EOF
 
   def self.generate_ocl_icd_loader_gen_source
     skip_funcs = $specific_loader_funcs
-    ocl_icd_loader_gen_source = "/**\n#{$license}\n*/\n"
-    ocl_icd_loader_gen_source += "#include <string.h>\n"
-    ocl_icd_loader_gen_source += "#include \"ocl_icd_loader.h\"\n"
-    ocl_icd_loader_gen_source += "#define DEBUG_OCL_ICD_PROVIDE_DUMP_FIELD\n"
-    ocl_icd_loader_gen_source += "#include \"ocl_icd_debug.h\"\n"
+    ocl_icd_loader_gen_source = <<EOF
+/**\n#{$license}\n*/
+#include <string.h>
+#include "ocl_icd_loader.h"
+#define DEBUG_OCL_ICD_PROVIDE_DUMP_FIELD
+#include "ocl_icd_debug.h"
+#define hidden_alias(name) \\
+  typeof(name) name##_hid __attribute__ ((alias (#name), visibility("hidden")))
+EOF
     api_proc = proc { |disp, (func_name, entry)|
       next if skip_funcs.include?(func_name)
       clean_entry = entry.sub(/(.*\)).*/m,'\1').gsub("/*","").gsub("*/","").gsub("\r","") + "{\n"
@@ -771,10 +794,22 @@ EOF
       #next if func_name.match(/EXT$/)
       #next if func_name.match(/KHR$/)
       if (skip_funcs.include?(func_name)) then
-        ocl_icd_loader_gen_source += "extern typeof(#{func_name}) #{func_name}_hid;\n"
-        ocl_icd_loader_gen_source += "extern typeof(#{func_name}) #{func_name}_disp;\n"
+        ocl_icd_loader_gen_source += <<EOF
+#if defined(__APPLE__) || defined(__MACOSX)
+#define #{func_name}_hid #{func_name}
+#else
+extern typeof(#{func_name}) #{func_name}_hid;
+#endif
+extern typeof(#{func_name}) #{func_name}_disp;
+EOF
       else
-        ocl_icd_loader_gen_source += "typeof(#{func_name}) #{func_name}_hid __attribute__ ((alias (\"#{func_name}\"), visibility(\"hidden\")));\n"
+        ocl_icd_loader_gen_source += <<EOF
+#if defined(__APPLE__) || defined(__MACOSX)
+#define #{func_name}_hid #{func_name}
+#else
+hidden_alias(#{func_name});
+#endif
+EOF
       end
     }
     ocl_icd_loader_gen_source += "\n\nstruct func_desc const function_description[]= {\n"
